#====================================
# Plugin: R: Arrayplots
# Date: 2003/08/14
# v: 0.92
#
# Array plots using Bioconductor(LIMMA) and SMA.
#
# Featured plots:
#
# 1) Imageplot
# 2) Background plots
# 3) Boxplot
# 4) MA-plot with print-tip loess curves
# 5) Densityplots
# 6) Overall Boxplots
#
# By: Ola Spjuth, ola@crossoverbio.com
# for The Linnaeus Centre for Bioinformatics, www.lcb.uu.se
#====================================

#Include functions for BASEfile read/write
source("/usr/local/base/plugins/R/parse.basefile.R")

#Disable warnings since they tend to scare some users :-(
options(warn=-1)

library(sma)
library(limma)

RG<-read.basefile("stdin.txt")
print("Data loaded")

#Set layout
layout<-list(ngrid.r=as.integer(ngrid.r.in),ngrid.c=as.integer(ngrid.c.in),nspot.r=as.integer(nspot.r.in),nspot.c=as.integer(nspot.c.in))

#Remove flagged spots
#=====================
if(do.flagremoval<0)
{
	print("Removing flagged spots")
	flags.na<-RG$Flags
	flags.na[flags.na>do.flagremoval]<-0
	flags.na[flags.na<=do.flagremoval]<-NA
	RG$R=RG$R*(flags.na+1)
	RG$G=RG$G*(flags.na+1)
}

#Include bg if needed
#Beware that MV are introduced if bgcorr

	RG.bg<-list(R=RG$R, G=RG$G, Rb=2*RG$Rb,Gb=2*RG$Gb)
	RG.trafo<-list(R=RG$R,G=RG$G)

if(do.bgcorr==1)
{
	print("Subtracting local background")
	RG.trafo<-RG.bg
}

#So, RG.bg always hold with background
#while RG.trafo only holds it if asked to

MAraw <- MA.RG(RG.trafo)		#Convert to MA (with or without bg)
no.assays<-dim(RG.trafo$R)[2]		#Number of Assays

#Create images for every array
for (ii in 1:no.assays)
{
	#Imageplot
	if(makeimageplots==1)
	{
		bitmap(file=paste("imgplot",ii,".png",sep=""), type = "png16m", height = 5, width = 14, res = 72)
		imageplot(MAraw$M[,ii],layout,zlim=c(-3,3), main="Imageplot")
		dev.off()
	}

	#Background-plots
	if(makebgplots==1)
	{
		bitmap(file=paste("Rbplot",ii,".png",sep=""), type = "png16m", height = 5, width = 14, res = 72)
		if (length(try(imageplot(log(RG.bg$Rb[,ii],2),layout, low="white", high="red", main="Plot of red background"),TRUE))>0)
			{print(paste("Error in Red BG-plot #",ii),sep="")}
		dev.off()
		bitmap(file=paste("Gbplot",ii,".png",sep=""), type = "png16m", height = 5, width = 14, res = 72)
		if (length(try(imageplot(log(RG.bg$Gb[,ii],2),layout, low="white", high="Green", main="Plot of green background"),TRUE))>0)
			{print(paste("Error in Green BG-plot #",ii),sep="")}
		dev.off()
	}

	#MA w lowesscurve
	if(makemaplots==1)
	{
		bitmap(file=paste("malowess",ii,".png",sep=""), type = "png16m", height = 7, width = 7, res = 72)
		if (length(try(plot.print.tip.lowess(RG.trafo,layout,pch=16,cex=0.2,image=ii, main="MA-plot with lowess curve"),TRUE))>0)
			{print(paste("Error in MA-plot #",ii,". Consider not removing flags and/or background."),sep="")}
		dev.off()
	}
	
	#Boxplot
	if(makeboxplots==1)
	{
		bitmap(file=paste("box",ii,".png",sep=""), type = "png16m", height = 7, width = 7, res = 72)
		plot.scale.box(MAraw$M[,3],layout,col=rainbow(layout$ngrid.r*layout$ngrid.c), main="Boxplot of print-tip groups")
		abline(0,0)
		dev.off()
	}
	
	#Densityplot
	if(makedensityplots==1)
	{
		bitmap(file=paste("density",ii,".png",sep=""), type = "png16m", height = 4, width = 4, res = 72)
		plot(density(MAraw$M[,ii],bw=0.17,na.rm=TRUE),xlim=range(-5,5),type = "l", col = "red", main=paste("Array #",ii,sep=""))
		abline(0,9999,0,col="blue")
		dev.off()
	}
}

	#Overall Boxplot
	bitmap(file="overallbox.png", type = "png16m", height = 6, width = 6, res = 72)
	boxplot(MAraw$M~col(MAraw$M),col=rainbow(no.assays))
	abline(0,0,col="blue")
	dev.off()

print("Result File")

#Result page as HTML
#===================
cat("<html><body>", file="Results.html")
cat("<h1>Results from R: Arrayplots</h1><p>", file="Results.html", append=TRUE)

for (ii in 1:no.assays)
{
	cat(paste("<p><h2>Array: ",ii,"</h2><br>",sep=""), file="Results.html", append=TRUE)
	if(makeimageplots==1)
	{
		cat(paste("<img src=\"imgplot",ii,".png\" ><p>",sep=""), file="Results.html", append=TRUE)
	}
	if(makebgplots==1)
	{
		cat(paste("<img src=\"Rbplot",ii,".png\" ><p>",sep=""), file="Results.html", append=TRUE)
		cat(paste("<img src=\"Gbplot",ii,".png\" ><p>",sep=""), file="Results.html", append=TRUE)
	}
	if(makemaplots==1)
	{
		cat(paste("<img src=\"malowess",ii,".png\" >",sep=""), file="Results.html", append=TRUE)
	}
	if(makeboxplots==1)
	{
		cat(paste("<img src=\"box",ii,".png\" ><p>",sep=""), file="Results.html", append=TRUE)
	}
}

for (ii in 1:no.assays)
{
	if(makedensityplots==1)
	{
		cat(paste("<img src=\"density",ii,".png\"> ",sep=""), file="Results.html", append=TRUE)
	}
}

cat("<p>", file="Results.html", append=TRUE)
cat("<h2>Overall boxplot of M-values for all arrays</h2><p>", file="Results.html", append=TRUE)
cat("<img src=\"overallbox.png\">", file="Results.html", append=TRUE)

cat("<p><hr><p>", file="Results.html", append=TRUE)
cat("Plots were obtained using R-libraries <a href=\"http://bioinf.wehi.edu.au/limma/\">Bioconductor(LIMMA)</a> and <a href=\"http://stat-www.berkeley.edu/users/terry/zarray/Software/smacode.html\">SMA</a>. <p>", file="Results.html", append=TRUE)
cat("(c) 2003 Ola Spjuth, <a href=\"http://www.crossoverbio.com\">www.crossoverbio.com <p>", file="Results.html", append=TRUE)

cat("</body></html>", file="Results.html", append=TRUE)

print("Finished")
