#====================================
# Plugin: R: B-Statistics
# Date: 2003/09/17
# v: 0.92
#
# Calculates B-statistics, t-statistics, and p-value for ranking genes
# Produces graphs and a ranked list of reporters based on B-statistics using Bioconductor(LIMMA)and SMA.
#
# By: Ola Spjuth, ola@crossoverbio.com
# for The Linnaeus Centre for Bioinformatics, www.lcb.uu.se
#====================================

#Include functions for BASEfile read/write
source("/usr/local/base/plugins/R/parse.basefile.R")

#Disable warnings since they tend to scare some users :-(
options(warn=-1)

#Load required libraries
library(sma)
library(limma)

#Read data from stdin
RG<-read.basefile.short.names("stdin.txt")
print("Data loaded")

#Convert input to design matrix
tmp<-as.numeric(unlist(strsplit(design.in,",")))
design<-matrix(nrow=as.numeric(ndesign.r.in), ncol=as.numeric(ndesign.c.in), data=tmp, byrow=TRUE)

RG.trafo<-list(R=RG$R,G=RG$G)
MAraw <- MA.RG(RG.trafo)			#Convert to MA
no.assays<-dim(RG.trafo$R)[2]			#Number of Assays

#Do B-stat

fit<-lm.series(MAraw$M,design)
eb<-ebayes(fit)
options(digits=3)

#Write ranked genelist to file
write.table(toptable(coef=2, number=dim(RG$pars)[1], RG$pars[,2:4], fit=fit, eb=eb, adjust="fdr"), sep="\t", file="RankedGenes.txt", row.names=FALSE, na="", quote=FALSE)

ord <- order(eb$lods[,2],decreasing=TRUE)
top <- ord[1:as.numeric(no.genesinplot)]

#Student's t Quantile-Quantile Plot
bitmap(file="qqt.png", type = "png16m", height = 12, width = 12, res = 72,)
	qqt(eb$t,df=eb$df+fit$df)
	abline(0,1)
dev.off()

#B-stat plot
bitmap(file="bstat.png", type = "png16m", height = 12, width = 12, res = 72,)
	plot(fit$coef[,2],eb$lods[,2], pch=16, cex=0.1, xlab="Log Fold Change", ylab="Log Odds")
	text(fit$coef[top,2],eb$lods[top,2],labels=as.character(RG$pars[top,4]),cex=0.7,col="blue")
dev.off()

#B-stat plot with reporterID
bitmap(file="bstat2.png", type = "png16m", height = 12, width = 12, res = 72,)
	plot(fit$coef[,2],eb$lods[,2], pch=16, cex=0.1, xlab="Log Fold Change", ylab="Log Odds")
	text(fit$coef[top,2],eb$lods[top,2],labels=as.character(RG$pars[top,3]),cex=0.7,col="blue")
dev.off()


#Result page as HTML
#===================
cat("<html><body>", file="Results.html")
cat("<h1>Results from B-statistics</h1>", file="Results.html", append=TRUE)
cat(" Below is the top 20 list of genes sorted by B-statistics. A complete list can be found in the file <a href=\"Rankedgenes.txt\">Rankedgenes.txt</a><p>", file="Results.html", append=TRUE)

cat("<table width=600 border=\"1\" frame=\"border\" rules=\"none\"><tr bgcolor=\"ddddff\"><td>", file="Results.html", append=TRUE)
write.table(toptable(coef=2, number=20, RG$pars[,2:4], fit=fit, eb=eb, adjust="fdr"), eol="</td><tr bgcolor=\"f0f0ff\">\n<td>",sep="</td><td>", file="Results.html", row.names=FALSE, na="", quote=FALSE, append=TRUE)
cat("</td></tr></table>", file="Results.html", append=TRUE)


cat("<h2>Student's t Quantile-Quantile Plot</h2>", file="Results.html", append=TRUE)
cat("<img src=\"qqt.png\"><p>", file="Results.html", append=TRUE)

cat("<h2>B-statistics plot with Gene Names as labels (NA are not included)</h2>", file="Results.html", append=TRUE)
cat("<img src=\"bstat.png\"><p>", file="Results.html", append=TRUE)

cat("<h2>B-statistics plot with reporterID as labels</h2>", file="Results.html", append=TRUE)
cat("<img src=\"bstat2.png\"><p>", file="Results.html", append=TRUE)

cat("<p><hr><p>", file="Results.html", append=TRUE)
cat("Normalization and Plots were obtained using R-libraries <a href=\"http://bioinf.wehi.edu.au/limma/\">Bioconductor(LIMMA)</a> and <a href=\"http://stat-www.berkeley.edu/users/terry/zarray/Software/smacode.html\">SMA</a>. <p>", file="Results.html", append=TRUE)
cat("(c) 2003 Ola Spjuth, <a href=\"http://www.crossoverbio.com\">www.crossoverbio.com <p>", file="Results.html", append=TRUE)

cat("</body></html>", file="Results.html", append=TRUE)

print("Finished")
