#====================================
# Plugin: R: Normalization
# Date: 2003/08/14
# v: 0.92
#
# Normalization using R functions from Bioconductor(LIMMA) and SMA.
#
# Featured normalization methods:
#
# 1) None
# 2) Median
# 3) Loess
# 4) Printtiploess
# 5) Composite
# 6) RobustSplines
#
# By: Ola Spjuth, ola@crossoverbio.com
# for The Linnaeus Centre for Bioinformatics, www.lcb.uu.se
#====================================

#Include functions for BASEfile read/write
source("/usr/local/base/plugins/R/parse.basefile.R")

#Disable warnings since they tend to scare some users :-(
options(warn=-1)

#Load required libraries
library(sma)
library(limma)

#Set tup list of norm methods
norm.methods<-list(l="Global lowess",n="None",c="Composite",p="Print-tip",m="Median",c="Composite",r="RobustSplines")

#Read data from stdin
RG<-read.basefile("stdin.txt")
print("Data loaded")

#Set up layout
layout<-list(ngrid.r=as.integer(ngrid.r.in),ngrid.c=as.integer(ngrid.c.in),nspot.r=as.integer(nspot.r.in),nspot.c=as.integer(nspot.c.in))


#Remove flagged spots
#=====================
if(do.flagremoval<0)
{
	print("Removing flagged spots with value =<")
	print(do.flagremoval)
	flags.na<-RG$Flags
	flags.na[flags.na>do.flagremoval]<-0
	flags.na[flags.na<=do.flagremoval]<-NA
	RG$R=RG$R*(flags.na+1)
	RG$G=RG$G*(flags.na+1)
}

#Include bg if needed
#Beware that MV are introduced if bgcorr

	RG.bg<-list(R=RG$R, G=RG$G, Rb=2*RG$Rb,Gb=2*RG$Gb)
	RG.trafo<-list(R=RG$R,G=RG$G)

if(do.bgcorr==1)
{
	print("Subtracting local background")
	RG.trafo<-RG.bg
}

#So, RG.bg always hold with background
#while RG.trafo only holds it if asked to

MAraw <- MA.RG(RG.trafo)		#Convert to MA
no.assays<-dim(RG$R)[2]			#Number of Assays

#Normalize
if (length(try(MA <- normalizeWithinArrays(RG.trafo,layout,method=norm.method),TRUE))==1)
{
	print("Error in normalization. Consider not removing flags and/or background.")
	stop("Normalization terminated.")	
}

#Inverse transform back to RG from MA
RG$R=round(2^(2*MA$A+MA$M))^(0.5)
RG$G=round(2^(2*MA$A-MA$M))^(0.5)

#Write output as transformation
write.transformation("stdout.txt",RG)

#Construct images
	bitmap(file="boxplot1.png", type = "png16m", height = 10, width = 10, res = 72)
		boxplot(MAraw$M~col(MAraw$M),col=rainbow(no.assays),names=paste("array",1:no.assays,sep=""),main="Before Normalization")
		abline(0,0,col="blue")
	dev.off()

	bitmap(file="boxplot2.png", type = "png16m", height = 10, width = 10, res = 72)
		boxplot(MA$M~col(MA$M),col=rainbow(no.assays),names=paste("array",1:no.assays,sep=""),main="After Normalization")
		abline(0,0,col="blue")
	dev.off()

#Result page as HTML
#===================
cat("<html><body>", file="Results.html")
cat(paste("<h1>Results from normalization using method: ",eval(parse(text=paste("norm.methods$",norm.method,sep=""))),"</h1><p>",sep=""), file="Results.html", append=TRUE)

cat("<img src=\"boxplot1.png\"><p>", file="Results.html", append=TRUE)
cat("<img src=\"boxplot2.png\"><p>", file="Results.html", append=TRUE)

cat("<p><hr><p>", file="Results.html", append=TRUE)
cat("Normalization and Plots were obtained using R-libraries <a href=\"http://bioinf.wehi.edu.au/limma/\">Bioconductor(LIMMA)</a> and <a href=\"http://stat-www.berkeley.edu/users/terry/zarray/Software/smacode.html\">SMA</a>. <p>", file="Results.html", append=TRUE)
cat("(c) 2003 Ola Spjuth, <a href=\"http://www.crossoverbio.com\">www.crossoverbio.com <p>", file="Results.html", append=TRUE)

cat("</body></html>", file="Results.html", append=TRUE)

print("Finished")
